#include "fontsize.h"
#include "util.h"
#include <windowsx.h>

static SIZE GetAlphaDimensions(HDC dc) {
 TCHAR buf[52],*p=buf,c;
 for (c='A';c<='Z';c++) *p++=c;
 for (c='a';c<='z';c++) *p++=c;
 SIZE sz;
 GetTextExtentPoint32(dc,buf,52,&sz);
// sz.cx=(sz.cx+26)/52;
 TEXTMETRIC tm;
 if (GetTextMetrics(dc,&tm)) sz.cy=tm.tmHeight;
 return sz;
}

SIZE font::MeasureFont(HWND hDlg,HFONT fnt) {
 HDC dc=GetDC(hDlg);
 HFONT ofnt=SelectFont(dc,fnt);
 SIZE sz=GetAlphaDimensions(dc);
 SelectFont(dc,ofnt);
 ReleaseDC(hDlg,dc);
 return sz;
}

BYTE font::GetFontBits(HWND w, HFONT font, LOGFONT&lf) {
 DBG(("LogFont: Height=%d, Weight=%d, Italic=%u, Pitch=%u, Family=%u, Name=<%s>",
	lf.lfHeight,
	lf.lfWeight,
	lf.lfItalic,
	lf.lfPitchAndFamily&15,
	lf.lfPitchAndFamily>>4,
	lf.lfFaceName));
 BYTE ft=0;
 ASSERT(lf.lfWeight==FW_BOLD||lf.lfWeight==FW_NORMAL);
 if (lf.lfWeight>400) ft|=1;
 if (lf.lfItalic) ft|=2;
 if (lf.lfPitchAndFamily&16) ft|=4;	// kurzum: FF_ROMAN(1<<4) und FF_MODERN(3<<4) haben Serifen
 if (lf.lfPitchAndFamily&9) ft|=8;	// kurzum: FIXED_PITCH(1) und MONO_FONT(8) haben feste Zeichenbreite
// Aus LOGFONT wird man ber Serifen und Monospace nicht schlau, wenn diese Werte auf 0 stehen.
// Da bleibt nichts anderes brig als den Font in einem DC zu realisieren und dann die Werte abzufragen.
 if (!(lf.lfPitchAndFamily&15) || !(lf.lfPitchAndFamily>>4)) {
  HDC dc=GetDC(w);
  HFONT fo=SelectFont(dc,font);
  TEXTMETRIC tm;
  BOOL r=GetTextMetrics(dc,&tm);
  ASSERT(r);
  ASSERT(lf.lfItalic==tm.tmItalic);
  if (!(tm.tmPitchAndFamily&1)) ft|=8;	// Anderes PITCH!!! Bit0 = Proportionalfont
// FF_MODERN impliziert normalerweise Courier und damit Serifen.
// Beim selbstgemachten Mono-Font fhrt das zum Fehler, daher per Stringvergleich prfen
  if (tm.tmPitchAndFamily&16 && lstrcmp(lf.lfFaceName,TEXT("Consolas"))) ft|=4;
  DBG(("TextMetric: Height=%d, AverageWidth=%d, Italic=%u, Pitch=%u, Family=%u",
	tm.tmHeight,
	tm.tmAveCharWidth,
	tm.tmItalic,
	tm.tmPitchAndFamily&15,
	tm.tmPitchAndFamily>>4));
  SelectFont(dc,fo);
  ReleaseDC(w,dc);
 }
 return ft;
}

HFONT font::CreateModifiedFont(HFONT font,UINT flags,UINT minmax) {
 LOGFONT lf;
 if (!font) font=GetStockFont(SYSTEM_FONT);
 if (!GetObject(font,sizeof lf,&lf)) return 0;
 BYTE ft=GetFontBits(0,font,lf);	// FontType
 DBG(("FontType detektiert:%X",ft));
 char sizechange=LOBYTE(flags);
 if (sizechange) {
  if (char(lf.lfHeight)<=char(LOBYTE(minmax))) return 0;	// gro(!) genug
  if (char(lf.lfHeight)>=char(HIBYTE(minmax))) return 0;
  lf.lfHeight=clamp(lf.lfHeight-sizechange,(char)LOBYTE(minmax),(char)HIBYTE(minmax));
  lf.lfWidth=0;
 }
 BYTE facechange=HIBYTE(flags);
 if (facechange) {
  if (facechange&0xF0) lf.lfFaceName[0]=0;	// Umschaltung Serifen/Monospace ohne Schriftname
  lf.lfOutPrecision=OUT_TT_PRECIS;
  lf.lfQuality=PROOF_QUALITY;
 }
 for (BYTE mask=1;facechange;mask<<=1,facechange>>=2) switch (facechange&3) {
  case 1: ft&=~mask;
  case 2: ft|= mask;
  case 3: ft^= mask;
 }
 DBG(("FontType gendert:%X",ft));
 lf.lfWeight=ft&1?FW_BOLD:FW_NORMAL;
 lf.lfItalic=ft&2?TRUE:FALSE;
// Korrektur:	FF_SWISS|FIXED_PITCH generiert Courier (= mit Serifen) = falsch!
//		FF_ROMAN|FIXED_PITCH generiert langgestrecktes China-Courier = hsslich!
 if (ft&8) {
  if (ft&4) {
   lf.lfPitchAndFamily=FF_MODERN|FIXED_PITCH;	// = Courier (mit Serifen)
  }else{
   lstrcpyn(lf.lfFaceName,TEXT("Consolas"),elemof(lf.lfFaceName));
   lf.lfPitchAndFamily=0;
  }
 }else lf.lfPitchAndFamily=BYTE(ft&4?FF_ROMAN|VARIABLE_PITCH:FF_SWISS|VARIABLE_PITCH);
 DBG(("New LogFont: Height=%d, Italic=%u, Pitch=%u, Family=%u, Name=<%s>",
	lf.lfHeight,
	lf.lfItalic,
	lf.lfPitchAndFamily&15,
	lf.lfPitchAndFamily>>4,
	lf.lfFaceName));
 return CreateFontIndirect(&lf);
}
